@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2007 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,16 +14,16 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py 3824 2008-04-08 22:29:11Z joanied $"
-__version__   = "$Revision: 3824 $"
-__date__      = "$Date: 2008-04-08 18:29:11 -0400 (Tue, 08 Apr 2008) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import bonobo
@@ -465,6 +465,20 @@ def __setupMagnifier(position, left=None, top=None, right=None, bottom=None,
     except:
         debug.printException(debug.LEVEL_WARNING)
 
+    # If we are running in full screen mode, try to hide the original cursor
+    # (assuming the user wants to). See bug #533095 for more details.
+    # Depends upon new functionality in gnome-mag, so just catch the 
+    # exception if this functionality isn't there.
+    #
+    hideCursor = restore.get('magHideCursor', settings.magHideCursor)
+    if hideCursor \
+       and _fullScreenCapable \
+       and _magnifier.SourceDisplay == _magnifier.TargetDisplay \
+       and position == settings.MAG_ZOOMER_TYPE_FULL_SCREEN:
+        hideSystemPointer(True)
+    else:
+        hideSystemPointer(False)
+
     _magnifierPBag = _magnifier.getProperties()
     sdb = _magnifierPBag.getValue("source-display-bounds").value()
     if not _originalSourceDisplayBounds:
@@ -989,6 +1003,7 @@ def shutdown():
     # See http://bugzilla.gnome.org/show_bug.cgi?id=375396.
     #
     try:
+        hideSystemPointer(False)
         _magnifier.clearAllZoomRegions()
         _magnifier.dispose()
     except:
@@ -1377,6 +1392,36 @@ def setZoomerColorFilter(colorFilter, updateScreen=True):
     if updateScreen:
         _zoomer.markDirty(_roi)
 
+def hideSystemPointer(hidePointer):
+    """Hide or show the system pointer.
+
+    Arguments:
+    -hidePointer: If True, hide the system pointer, otherwise show it.
+    """
+
+    # Depends upon new functionality in gnome-mag, so just catch the
+    # exception if this functionality isn't there.
+    #
+    try:
+        if hidePointer:
+            _magnifier.hideCursor()
+        else:
+            _magnifier.showCursor()
+    except:
+        debug.printException(debug.LEVEL_FINEST)
+
+def isFullScreenCapable():
+    """Returns True if we are capable of doing full screen (i.e. whether
+    composite is being used.
+    """
+
+    try:
+        capable = _magnifier.fullScreenCapable()
+    except:
+        capable = False
+
+    return capable
+
 def isFilteringCapable():
     """Returns True if we're able to take advantage of libcolorblind's color
     filtering.
