@@ -61,16 +61,26 @@ import signal
 import time
 import unicodedata
 
+import settings
+if settings.useDBus:
+    import dbus.mainloop.glib
+    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
+    import dbusserver
+    try:
+        import gsmag as mag
+    except:
+        import mag
+else:
+    import mag
+
 import pyatspi
 import braille
 import debug
 import httpserver
 import keynames
 import keybindings
-import mag
 import orca_state
 import platform
-import settings
 import speech
 
 from input_event import BrailleEvent
@@ -78,9 +88,6 @@ from input_event import KeyboardEvent
 from input_event import MouseButtonEvent
 from input_event import keyEventToString
 
-if settings.useDBus:
-    import dbusserver
-
 from orca_i18n import _           # for gettext support
 
 if settings.debugMemoryUsage:
@@ -1532,11 +1539,16 @@ def usage():
     print "-q, --quit                   " + \
           _("Quits Orca (if shell script used)")
 
-    print
+    # Translators: this is the Orca command line option to tell Orca to
+    # replace any existing Orca process(es) that might be running.
+    #
+    print "--replace                    " +\
+          _("Replace a currently running Orca")
 
     # Translators: this is text being sent to a terminal and we want to
     # keep the text lines within terminal boundaries.
     #
+    print
     print _("If Orca has not been previously set up by the user, Orca\n" \
             "will automatically launch the preferences set up unless\n" \
             "the -n or --no-setup option is used.")
@@ -1614,6 +1626,7 @@ def main():
         # n is for no setup
         # t is for text setup
         # v is for version
+        # z is for no exit
         #
         opts, args = getopt.getopt(
             arglist,
@@ -1629,7 +1642,8 @@ def main():
              "list-apps",
              "debug",
              "debug-file=",
-             "version"])
+             "version",
+             "replace"])
         for opt, val in opts:
             if opt in ("-u", "--user-prefs-dir"):
                 userPrefsDir = val.strip()
