@@ -1,4 +1,4 @@
-# Copyright 2006, 2007, 2008 Brailcom, o.p.s.
+# Copyright 2006, 2007, 2008, 2009 Brailcom, o.p.s.
 #
 # Author: Tomas Cerha <cerha@brailcom.org>
 #
@@ -31,9 +31,9 @@
 NOTE: THIS IS EXPERIMENTAL ONLY AND IS NOT A SUPPORTED COMPONENT OF ORCA.
 """
 
-__id__ = "$Id: speechdispatcherfactory.py 4433 2009-01-19 15:46:14Z wwalker $"
-__version__   = "$Revision: 4433 $"
-__date__      = "$Date: 2009-01-19 10:46:14 -0500 (Mon, 19 Jan 2009) $"
+__id__ = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __author__    = "Tomas Cerha <cerha@brailcom.org>"
 __copyright__ = "Copyright (c) 2006-2008 Brailcom, o.p.s."
 __license__   = "LGPL"
@@ -90,10 +90,12 @@ class SpeechServer(speechserver.SpeechServer):
     getFactoryName = staticmethod(getFactoryName)
 
     def getSpeechServers():
+        servers = []
         default = SpeechServer._getSpeechServer(SpeechServer.DEFAULT_SERVER_ID)
-        servers = [default]
-        for module in default.list_output_modules():
-            servers.append(SpeechServer._getSpeechServer(module))
+        if default is not None:
+            servers.append(default)
+            for module in default.list_output_modules():
+                servers.append(SpeechServer._getSpeechServer(module))
         return servers
     getSpeechServers = staticmethod(getSpeechServers)
 
@@ -240,6 +242,16 @@ class SpeechServer(speechserver.SpeechServer):
                 current[acss_property] = value
 
     def _speak(self, text, acss, **kwargs):
+        # Replace no break space characters with plain spaces since some
+        # synthesizers cannot handle them.  See bug #591734.
+        #
+        text = text.decode("UTF-8").replace(u'\u00a0', ' ').encode("UTF-8")
+
+        # Replace newline followed by full stop, since
+        # this seems to crash sd, see bgo#618334.
+        #
+        text = text.replace('\n.', '\n')
+
         self._apply_acss(acss)
         self._send_command(self._client.speak, text, **kwargs)
 
